package cn.efunbox.manage.base.service.impl;

import cn.efunbox.manage.base.entity.User;
import cn.efunbox.manage.base.enums.BaseStatusEnum;
import cn.efunbox.manage.base.repository.UserRepository;
import cn.efunbox.manage.base.service.UserService;
import cn.efunbox.manage.base.vo.UserVo;
import cn.efunbox.manage.common.result.ApiCode;
import cn.efunbox.manage.common.result.ApiResult;
import cn.efunbox.manage.common.utils.BeanUtil;
import cn.efunbox.manage.common.utils.MD5;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;

@Service
public class UserServiceImpl implements UserService {

    @Autowired
    private UserRepository userRepository;


    @Override
    public ApiResult<UserVo> login(String userName, String password) {
        if(StringUtils.isBlank(userName)||StringUtils.isBlank(password)){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        User User = userRepository.findByUserNameAndStatus(userName, BaseStatusEnum.NORMAL);
        String MD5Password = User.getPassword();
        if(User==null){
            return ApiResult.error(ApiCode.NO_UID);
        }
        if(StringUtils.isBlank(MD5Password)|| !MD5Password.equals(MD5.MD5Encode(password))){
            return ApiResult.error(ApiCode.PASSWORD_ERROR);
        }
        UserVo UserVo = new UserVo();
        BeanUtil.copy(User,UserVo);
        return ApiResult.ok(UserVo);
    }

    @Override
    public ApiResult<User> saveUser(User User) {

        String userName = User.getUserName();
        String password = User.getPassword();
        if(User==null||StringUtils.isBlank(userName)||StringUtils.isBlank(password)){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        User repeatUser = userRepository.findByUserNameAndStatus(userName, BaseStatusEnum.NORMAL);
        if(repeatUser!=null){
            return ApiResult.error(ApiCode.USERNAME_REPEAT);
        }
        User.setStatus(BaseStatusEnum.NORMAL);
        User.setPassword(MD5.getMD5Str(password));
        userRepository.save(User);
        return ApiResult.ok(User);
    }

    @Override
    public ApiResult<User> updateUser(User User) {

        if(User==null||StringUtils.isBlank(User.getId())){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        String password = User.getPassword();
        if(StringUtils.isNotBlank(password)){
            User.setPassword(MD5.getMD5Str(password));
        }
        userRepository.update(User);
        return ApiResult.ok(User);
    }
    @Transactional
    @Override
    public ApiResult<User> deleteUserByIds(List<String> ids) {
        if(ids==null||ids.size()==0){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        userRepository.updateStatusByIds(ids,BaseStatusEnum.DEL);
        return ApiResult.ok();
    }
}
