/*
*
 * 文件名：@WebConfiguration.java <br/>
 * 包名：tv.acfun.album.front.configuration <br/>
 * 项目名：acfun-album-front <br/>
 * @author tomas <br/>

*/

package cn.efunbox.manage.common.configuration;


import cn.efunbox.manage.common.intercepter.AdminInterceptor;
import cn.efunbox.manage.common.intercepter.AllowOriginInterceptor;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.alibaba.fastjson.support.config.FastJsonConfig;
import com.alibaba.fastjson.support.spring.FastJsonHttpMessageConverter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.EnvironmentAware;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;

import javax.validation.Validation;
import javax.validation.Validator;
import java.util.ArrayList;
import java.util.List;

/**
 * 类名：WebConfiguration  <br />
 *
 * 功能：Web相关配置
 *
 * @author tomas <br />
 * 创建时间：2016年7月27日 下午3:57:19  <br />
 * @version 2016年7月27日*/
@Configuration
@EnableWebMvc
public class FrontConfiguration extends WebMvcConfigurerAdapter implements EnvironmentAware {
	// 日志记录器
	private static final Logger logger = LoggerFactory.getLogger(FrontConfiguration.class);

	// 当前的环境对象
	protected Environment environment;
	@Autowired
	AllowOriginInterceptor allowOriginInterceptor;

	@Autowired
	private AdminInterceptor adminInterceptor;

	public void addInterceptors(InterceptorRegistry registry) {

		registry.addInterceptor(allowOriginInterceptor);
		registry.addInterceptor(adminInterceptor)
				.addPathPatterns("/cms/**")
				.excludePathPatterns("/cms/login")
				.excludePathPatterns("/cms/logout");
	}

    @Override
    public void configureMessageConverters(List<HttpMessageConverter<?>> converters) {
        super.configureMessageConverters(converters);

//        1.需要先定义一个convert转换消息的对象；
//        2.添加fastjson的配置信息，比如是否要格式化返回的json数据
//        3.在convert中添加配置信息
//        4.将convert添加到converters中

        //1.定义一个convert转换消息对象
        FastJsonHttpMessageConverter fastConverter=new FastJsonHttpMessageConverter();
        List<MediaType> mediaTypeList = new ArrayList<>();
        mediaTypeList.add(MediaType.APPLICATION_JSON);
        mediaTypeList.add(MediaType.TEXT_HTML);
        mediaTypeList.add(MediaType.APPLICATION_FORM_URLENCODED);

        fastConverter.setSupportedMediaTypes(mediaTypeList);
        //2.添加fastjson的配置信息，比如：是否要格式化返回json数据
        FastJsonConfig fastJsonConfig=new FastJsonConfig();
        fastJsonConfig.setSerializerFeatures(
                SerializerFeature.PrettyFormat,
				SerializerFeature.DisableCircularReferenceDetect,
                SerializerFeature.WriteNullListAsEmpty,
                SerializerFeature.WriteMapNullValue,
                SerializerFeature.WriteNullStringAsEmpty
        );
        fastConverter.setFastJsonConfig(fastJsonConfig);
        converters.add(fastConverter);
    }

	@Bean("validator")
	public Validator validator() {
		return Validation.buildDefaultValidatorFactory().getValidator();
	}

   /* @Bean
    public HttpMessageConverters fastJsonHttpMessageConverters() {

        FastJsonHttpMessageConverter fastConverter = new FastJsonHttpMessageConverter();

        FastJsonConfig fastJsonConfig = new FastJsonConfig();

        fastJsonConfig.setSerializerFeatures(
//                SerializerFeature.PrettyFormat,
                SerializerFeature.WriteNullListAsEmpty,
                SerializerFeature.WriteMapNullValue,
                SerializerFeature.WriteNullStringAsEmpty);

        fastConverter.setFastJsonConfig(fastJsonConfig);

        HttpMessageConverter<?> converter = fastConverter;

        return new HttpMessageConverters(converter);

    }*/

	/**
	 * Set the {@code Environment} that this object runs in.
	 *
	 * @param environment
	 */
	@Override
	public void setEnvironment(Environment environment) {
		this.environment = environment;
	}
}